@@ -39,6 +39,7 @@ Lisp_Object Qcall_interactively;
 Lisp_Object Vcommand_history;
 
 extern Lisp_Object Vhistory_length;
+extern Lisp_Object real_this_command;
 
 Lisp_Object Vcommand_debug_status, Qcommand_debug_status;
 Lisp_Object Qenable_recursive_minibuffers;
@@ -222,6 +223,13 @@ supply if the command inquires which events were used to invoke it.")
   struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
   int key_count;
 
+  Lisp_Object save_this_command, save_last_command;
+  Lisp_Object save_real_this_command;
+
+  save_this_command = Vthis_command;
+  save_real_this_command = real_this_command;
+  save_last_command = current_kboard->Vlast_command;
+
   if (NILP (keys))
     keys = this_command_keys, key_count = this_command_key_count;
   else
@@ -363,6 +371,11 @@ supply if the command inquires which events were used to invoke it.")
 		XCDR (teml) = Qnil;
 	    }
 	}
+
+      Vthis_command = save_this_command;
+      real_this_command= save_real_this_command;
+      current_kboard->Vlast_command = save_last_command;
+
       single_kboard_state ();
       return apply1 (function, specs);
     }
@@ -788,6 +801,10 @@ supply if the command inquires which events were used to invoke it.")
     if (varies[i] >= 1 && varies[i] <= 4)
       XSETINT (args[i], marker_position (args[i]));
 
+  Vthis_command = save_this_command;
+  real_this_command= save_real_this_command;
+  current_kboard->Vlast_command = save_last_command;
+
   single_kboard_state ();
 
   {
