@@ -784,7 +784,13 @@ mw32_BDF_TextOut (bdffont *fontp, HDC hdc, int left,
 	      DIBsection_hdc = hdc;
 	      DIBsection_width = width;
 	      DIBsection_height = height;
-	      if (!hBMP) return 0;
+	      if (!hBMP)
+		{
+		  DeleteDC (hCompatDC);
+		  SelectObject (hdc, hOrgBrush);
+		  DeleteObject (hFgBrush);
+		  return 0;
+		}
 	    }
 
 	  memcpy (bits, pcb->pbmp, pcb->bitmap_size);
