@@ -272,6 +272,7 @@ mw32_set_layout (MW32LogicalFont *plf, HDC hdc, unsigned char *pstr,
 		 int nbytes, int maxextent, int *xoffs,
 		 int *pleft_ov, int *pright_ov)
 {
+#if 0 /* No layout information is used currently. */
   int result;
   HANDLE hold;
   GCP_RESULTS gcp;
@@ -320,6 +321,9 @@ mw32_set_layout (MW32LogicalFont *plf, HDC hdc, unsigned char *pstr,
   SelectObject (hdc, hold);
 
   return 1;
+#else
+  return 0;
+#endif
 }
 
 static void mw32_logfont_free (MW32LogicalFont *plf)
@@ -407,6 +411,17 @@ mw32_set_windows_logical_font (struct frame *f, MW32LogicalFont *plf,
 	  xfree (pwf);
 	  return 0;
 	}
+      /* If the font width is not desired, adjust `lfWidth' and
+	 recreate it later. */
+      if (plogf->lfWidth != 0 && plogf->lfWidth != tm.tmAveCharWidth)
+	{
+	  int target_width = plogf->lfWidth;
+	  plogf->lfWidth = plogf->lfWidth * plogf->lfWidth / tm.tmAveCharWidth;
+	  tm.tmAveCharWidth = target_width;
+	  DeleteObject (hf);
+	  hf = INVALID_HANDLE_VALUE;
+	}
+      
       pwf->pfont = hf;
 
       plf->width = tm.tmAveCharWidth;
@@ -1519,20 +1534,6 @@ mw32_compute_min_glyph_bounds (FRAME_PTR f)
 	  || minheight < old_minheight);
 }
 
-static LOGFONT default_logfont_courier_new =
-{
-  0, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
-  OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
-  FF_DONTCARE | FIXED_PITCH, "Courier New"
-};
-
-static LOGFONT default_logfont_fixed_sys =
-{
-  0, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
-  OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
-  FF_DONTCARE | FIXED_PITCH, "FixedSys"
-};
-
 static MW32LogicalFont*
 mw32_add_logical_font (MW32FontRequest *pfr)
 {
@@ -1718,6 +1719,25 @@ mw32_face_family_unspecified_p (Lisp_Object family_attr)
   return 0;
 }
 
+static char mw32_enumerated_face_name[LF_FACESIZE];
+
+static int CALLBACK
+enumfont_callback (ENUMLOGFONTEX *elogfont,
+		   NEWTEXTMETRICEX *metric,
+		   DWORD font_type,
+		   LPARAM lparam)
+{
+  if (metric->ntmTm.tmPitchAndFamily & TMPF_TRUETYPE
+      && !(metric->ntmTm.tmPitchAndFamily & TMPF_FIXED_PITCH)
+      && elogfont->elfLogFont.lfFaceName[0] != '@')
+    {
+      memcpy (mw32_enumerated_face_name,
+	      elogfont->elfLogFont.lfFaceName, LF_FACESIZE);
+      return 0;
+    }
+  return 1;
+}
+
 static int
 mw32_convert_lface_to_LOGFONT (struct frame *f,
 			       Lisp_Object *attrs, LOGFONT *plogf,
@@ -1743,9 +1763,6 @@ mw32_convert_lface_to_LOGFONT (struct frame *f,
       plogf->lfCharSet = ANSI_CHARSET;
     }
 
-  if (!mw32_face_family_unspecified_p (attrs[LFACE_FAMILY_INDEX]))
-    encode_logfont_name (attrs[LFACE_FAMILY_INDEX], plogf->lfFaceName);
-
   if (!UNSPECIFIEDP (attrs[LFACE_HEIGHT_INDEX]))
     {
       Lisp_Object elt = attrs[LFACE_HEIGHT_INDEX];
@@ -1754,7 +1771,9 @@ mw32_convert_lface_to_LOGFONT (struct frame *f,
 	pt = XINT (elt);
       else if (FLOATP (elt))
 	pt = (int) XFLOATINT (elt);
-      plogf->lfHeight = (int)(- pt * FRAME_MW32_DISPLAY_INFO (f)->resy / 720);
+      /* I want this value to be a cell height that includes internal
+	 leading, so this should be positive. */
+      plogf->lfHeight = (int)(pt * FRAME_MW32_DISPLAY_INFO (f)->resy / 720);
     }
 
   if (!UNSPECIFIEDP (attrs[LFACE_WEIGHT_INDEX]))
@@ -1805,13 +1824,42 @@ mw32_convert_lface_to_LOGFONT (struct frame *f,
       else if (EQ (Qreverse_oblique, elt))
 	plogf->lfItalic = 0;  /* TODO */
     }
+
+  if (!mw32_face_family_unspecified_p (attrs[LFACE_FAMILY_INDEX]))
+    encode_logfont_name (attrs[LFACE_FAMILY_INDEX], plogf->lfFaceName);
+  else
+    {
+      HWND hwnd = GetDesktopWindow ();
+      HDC hdc = GetDC (hwnd);
+      mw32_enumerated_face_name[0] = '\0';
+      EnumFontFamiliesEx (hdc, plogf, (FONTENUMPROC) enumfont_callback, 0, 0);
+      ReleaseDC (hwnd, hdc);
+
+      if (mw32_enumerated_face_name[0] != '\0')
+	memcpy (plogf->lfFaceName, mw32_enumerated_face_name, LF_FACESIZE);
+      else
+	{
+	  plogf->lfHeight = plogf->lfWidth = 0;
+	  strncpy (plogf->lfFaceName, "FixedSys", LF_FACESIZE);
+	}
+    }
+
   return 1;
 }
 
 void
 mw32_initialize_default_logfont (LOGFONT* lf)
 {
-  *lf = default_logfont_fixed_sys;
+  static LOGFONT default_logfont =
+    {
+      /* The height is explicitly set here to avoid the annoying
+	 variable font width. */
+      16, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
+      OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
+      FF_DONTCARE | FIXED_PITCH, ""
+    };
+
+  *lf = default_logfont;
 }
 
 static MW32LogicalFont*
@@ -1831,19 +1879,12 @@ mw32_default_logical_font_loader (MW32FontRequest *pfr,
 
   plf = mw32_add_logical_font (pfr);
 
-  /* first, try `courier new' */
-  lf = default_logfont_courier_new;
+  mw32_initialize_default_logfont (&lf);
   if ((!mw32_convert_lface_to_LOGFONT (f, attrs, &lf, c))
       || (!mw32_set_windows_logical_font (f, plf, &lf, c)))
     {
-      /* next, try `FixedSys' */
-      lf = default_logfont_fixed_sys;
-      if ((!mw32_convert_lface_to_LOGFONT (f, attrs, &lf, c))
-	  || (!mw32_set_windows_logical_font (f, plf, &lf, c)))
-	{
-	  mw32_delete_logical_font (plf);
-	  return NULL;
-	}
+      mw32_delete_logical_font (plf);
+      return NULL;
     }
 
   return mw32_register_logical_font (f, plf);
@@ -2222,10 +2263,11 @@ void set_font_frame_param (frame, lface)
 void
 mw32font_set_frame_ime_font_by_llogfont (FRAME_PTR f, Lisp_Object llf)
 {
+#ifdef IME_CONTROL
   LOGFONT lf;
 
   if (NILP (llf))
-    lf = default_logfont_fixed_sys;
+    mw32_initialize_default_logfont (&lf);
   else
     lf = lisp_object_to_logfont (llf);
   f->output_data.mw32->ime_logfont = lf;
@@ -2233,6 +2275,7 @@ mw32font_set_frame_ime_font_by_llogfont (FRAME_PTR f, Lisp_Object llf)
 		       WM_MULE_IMM_SET_COMPOSITION_FONT,
 		       (WPARAM) f,
 		       (LPARAM) &lf);
+#endif /* IME_CONTROL */
 }
 
 
@@ -2416,7 +2459,7 @@ DEFUN ("w32-font-list", Fw32_font_list, Sw32_font_list, 0, 0, 0,
 
 DEFUN ("w32-get-font-info", Fw32_get_font_info,
        Sw32_get_font_info, 1, 1, 0,
-       "Get the font infomation you specified.\n\
+       "Get the font information you specified.\n\
 NAME is a name of the font.")
      (name)
 {
