@@ -837,6 +837,7 @@ struct scroll_bar
 #define WM_EMACS_SETCARET                          (WM_USER+2009)
 #define WM_EMACS_CLEAR_MOUSE_FACE                  (WM_USER+2010)
 #define WM_EMACS_CREATE_TIP_FRAME                  (WM_USER+2011) 
+#define WM_EMACS_SETFOREGROUND                     (WM_USER+2012)
 /* to report "switch-buffer" event by himi */
 #define WM_IME_REPORT     0x0280
 #define IR_STRING         0x140
@@ -1220,4 +1221,12 @@ typedef struct tagTRACKMOUSEEVENT
 } TRACKMOUSEEVENT, *LPTRACKMOUSEEVENT;
 #endif
 
+#ifndef SPI_GETFOREGROUNDLOCKTIMEOUT
+#define SPI_GETFOREGROUNDLOCKTIMEOUT        0x2000
+#endif
+
+#ifndef SPI_SETFOREGROUNDLOCKTIMEOUT
+#define SPI_SETFOREGROUNDLOCKTIMEOUT        0x2001
+#endif
+
 #endif /* not _MW32TERM_H_ */
