@@ -741,6 +741,17 @@ sys_spawnve (int mode, char *cmdname, char **argv, char **envp)
   char *extra_env[] = {ppid_env_var_buffer, NULL};
   char *sepchars = " \t";
 
+#ifdef MEADOW
+  /* duplicate the whole elements of `argv' to prevent errors in GC.
+   */
+  targ = argv;
+  while (*targ)
+    {
+      parg = alloca (strlen (*targ) + 1);
+      strcpy (parg, *targ);
+      *targ++ = parg;
+    }
+#endif
   /* We don't care about the other modes */
   if (mode != _P_NOWAIT)
     {
@@ -780,7 +791,13 @@ sys_spawnve (int mode, char *cmdname, char **argv, char **envp)
 #endif
 
   /* make sure argv[0] and cmdname are both in DOS format */
+#ifdef MEADOW
+  /* alloc new memory to cmdname to make argv[0] be GC-safe */
+  cmdname = (char *) alloca (SBYTES (program) + 1);
+  strcpy (cmdname, SDATA (program));
+#else
   cmdname = XSTRING (program)->data;
+#endif
 
   unixtodos_filename (cmdname);
   argv[0] = cmdname;
